///////////////////////////////////////////////////////////////////////////////
/*! \file Lighting.h
 *  \brief Framework Nemo
 *  \author C. Crassin
 *
 * $Id: Timer.h,v 1.2 2007/09/14 11:54:56 gascuel Exp $
 */
///////////////////////////////////////////////////////////////////////////////
// History:
//
// 2005-01-01 : Creation.

#ifndef NEMO_Timer_h
#define NEMO_Timer_h

#ifdef WIN32
#include <windows.h>
#else
#   include <time.h>
#endif

#include <NemoGraphics/UtilsGL.h>
#include <NemoGraphics/MessageHandler.h>
#include <vector>
#include <string>
#include <iomanip>
#include <iostream>

namespace NemoGraphics {

extern double globalTimerStat[16];
extern long globalTimerStatNum[16];
extern bool globalTimerStarted;

#ifdef WIN32
extern LARGE_INTEGER ticksPerSecond;
extern LARGE_INTEGER globalTimerTick;   // A point in time
#else
extern struct timespec globalStartTime;
#endif


extern std::vector<double> globalTimerStack;
extern std::vector<std::string> globalTimerMsgStack;

inline bool GlobalTimerStarted(){
	return globalTimerStarted;
}
inline void StartGlobalTimer(){
#ifdef WIN32
	QueryPerformanceCounter(&globalTimerTick);
	QueryPerformanceFrequency(&ticksPerSecond);
#else
	clock_gettime(CLOCK_REALTIME, &globalStartTime);
#endif
	globalTimerStack.clear();

}

inline double GetGlobalTimer(){
#ifdef WIN32
	LARGE_INTEGER lEnd;
	QueryPerformanceCounter(&lEnd);

	return (double(lEnd.QuadPart - globalTimerTick.QuadPart) / (double)ticksPerSecond.QuadPart);
#else
    struct timespec stopTime;
    clock_gettime(CLOCK_REALTIME, &stopTime);
    return (stopTime.tv_sec  - globalStartTime.tv_sec ) * 1000.0f // sec -> msec
         + (stopTime.tv_nsec - globalStartTime.tv_nsec) * 1e-6f;  // nano -> milli
#endif

}

inline void PushGlobalTimerGL(const std::string &msg){
#ifdef NEMO_TIMERS_ON
	glFlush();
	glFinish();

	globalTimerStack.push_back(GetGlobalTimer());
	globalTimerMsgStack.push_back(msg);
#endif
}
inline void DisplayGlobalTimerGL(){
#ifdef NEMO_TIMERS_ON
	int i=0;

	std::vector<double>::iterator it;
	std::vector<std::string>::iterator itm;

	for(it=globalTimerStack.begin(), itm=globalTimerMsgStack.begin(); 
		it!=globalTimerStack.end(); it++, itm++){
		std::cout<<(*itm)<<" -> "<<(*it)<<" , ";
		i++;
	}
	std::cout<<"\n";
#endif
}

inline void StartGlobalTimerStat(int n){
	globalTimerStatNum[n]=0;
	globalTimerStat[n]=0.0;
}
inline void UpdateGlobalTimerStat(int n){
#ifdef WIN32
	QueryPerformanceFrequency(&ticksPerSecond);

	LARGE_INTEGER lEnd;
	QueryPerformanceCounter(&lEnd);

	globalTimerStat[n]+=(double(lEnd.QuadPart - globalTimerTick.QuadPart) / (double)ticksPerSecond.QuadPart);
#else

	 struct timespec stopTime;
   clock_gettime(CLOCK_REALTIME, &stopTime);
   globalTimerStat[n]+= (stopTime.tv_sec  - globalStartTime.tv_sec ) * 1000.0f // sec -> msec
         + (stopTime.tv_nsec - globalStartTime.tv_nsec) * 1e-6f;  // nano -> milli

#endif
	globalTimerStatNum[n]++;
}
inline double GetGlobalTimerStat(int n){
	return globalTimerStat[n]/(double)globalTimerStatNum[n];
}


/////////////////////////NEW CLASS////////////////////////////

class PerfTimer {
#ifdef WIN32
	LARGE_INTEGER ticksPerSecond;
	LARGE_INTEGER globalTimerTick;   // A point in time
#else
	extern struct timespec globalStartTime;
#endif

	std::vector<double> globalTimerStack;
	std::vector<std::string> globalTimerMsgStack;

	static int globalState;


	////GPU times

public:
	PerfTimer(){
		QueryPerformanceFrequency(&this->ticksPerSecond);
	}

	static void globalEnable(){
		globalState=true;
	}
	static void globalDisable(){
		globalState=false;
	}

	void start(){
		if(globalState){
#ifdef WIN32
			QueryPerformanceCounter(&this->globalTimerTick);
#else	
			clock_gettime(CLOCK_REALTIME, &globalStartTime);
#endif

			globalTimerStack.clear();
		}
	}

	double getTime(){
		if(globalState){
#ifdef WIN32
			LARGE_INTEGER lEnd;
			QueryPerformanceCounter(&lEnd);

			return (double(lEnd.QuadPart - this->globalTimerTick.QuadPart) / (double)this->ticksPerSecond.QuadPart);
#else

			struct timespec stopTime;
			clock_gettime(CLOCK_REALTIME, &stopTime);
			return (stopTime.tv_sec  - globalStartTime.tv_sec ) * 1000.0f // sec -> msec
					 + (stopTime.tv_nsec - globalStartTime.tv_nsec) * 1e-6f;  // nano -> milli

#endif
		}else{
			return 0.0;
		}

	}


	inline void pushTime(const std::string &msg, bool restart=false){
		if(globalState){
			glFlush();
			glFinish();

			this->globalTimerStack.push_back(this->getTime());
			this->globalTimerMsgStack.push_back(msg);

			if(restart){
#ifdef WIN32
				QueryPerformanceCounter(&this->globalTimerTick);
#else
				clock_gettime(CLOCK_REALTIME, &globalStartTime);
#endif
			}
		}
	}

	inline void displayTimes(const std::string &msg){
		if(globalState){
			int i=0;

			std::vector<double>::iterator it;
			std::vector<std::string>::iterator itm;

			std::cout<<msg<<" :\n";
			for(it=this->globalTimerStack.begin(), itm=this->globalTimerMsgStack.begin(); 
				it!=this->globalTimerStack.end(); it++, itm++){
					std::cout<<(*itm)<<" ->"<<std::setw(15) <<(*it)<<" , ";
				i++;
			}
			std::cout<<"\n";
		}
	}

};

};

#endif

